from websocket_server import WebsocketServer
from datapoint import *
import json
from iaqobject import *
from os import listdir
from os.path import isfile, join
from importlib import import_module
import sys

from settings import general
from settings import wood
from settings import copper

sys.path.append('./')

objects = []

# Called for every client connecting (after handshake)
def new_client(client, server):
	print("New client connected and was given id %d" % client['id'])
	client['objects'] = []
	server.send_message_to_all("Hey all, a new client has joined us")


# Called for every client disconnecting
def client_left(client, server):
	print("Client(%d) disconnected" % client['id'])
	client['objects'] = []


def create_object(d, client):
	o = IaqObject(d['name'], d['property'], d['x'], d['y'])

	client['objects'].append(o)

def delete_object(d, client):
	for o in client['objects']:
		if (o.name == d['name']):
			client['objects'].remove(o)

			break

def move_object(d, client):
	for o in client['objects']:
		if (o.name == d['name']):
			o.x = d['x']
			o.y = d['y']

			break

def rename_object(d, client):
	for o in client['objects']:
		if (o.name == d['name']):
			o.name = d['new_name']

			break

def send_files(client, server):
	mypath = "html/uploads"
	onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]

	files = ""
	for f in onlyfiles:
		files += "\"" + f + "\""
		files += ","

	files = files[:-1]

	msg = "{\"action\" : \"files\", \"data\": [ " + files + "] }"
	server.send_message(client, msg)

def send_settings(client, server, name):
	file = open("settings/" + name + ".py", "r")

	line = file.readline()

	settings = "{"

	i = 1

	while (line):
		line = file.readline()

		if ("[" in line):
			parts = line.split("[")[1]
			part = parts.split("]")[0]

			values = part.split(", ")

			settings += "\"t" + str(i) + "\" : \"" + values[0] + "\","
			settings += "\"rh" + str(i) + "\" : \"" + values[1] + "\","

			i += 1

	settings = settings[:-1]
	settings += "}"

	msg = "{\"action\" : \"settings\", \"data\": [ " + settings + "] }"

	file.close() 

	server.send_message(client, msg)

def save_settings(name, data, s_type):

	print data
	file = open("settings/" + name + ".py", "r")

	line = file.readline()

	temp = []
	rh = []

	while (line):
		line = file.readline()

		if ("[" in line):
			parts = line.split("[")[1]
			part = parts.split("]")[0]

			values = part.split(", ")

			temp.append(values[0])
			rh.append(values[1])

	file.close()

	if s_type == "temp":
		temp  = data
	elif s_type == "rh":
		rh = data

	file = open("settings/" + name + ".py","w")

	file.write('parameter_names = ["Temperature", "Relative humidity"]\n')
	file.write('parameter_lower_bounds = [' + str(temp[0]) + ', ' + str(rh[0]) + ']\n')
	file.write('parameter_lower_bounds1 = [' + str(temp[1]) + ', ' + str(rh[1]) + ']\n')
	file.write('parameter_upper_bounds = [' + str(temp[2]) + ', ' + str(rh[2]) + ']\n')
	file.write('parameter_upper_bounds1 = [' + str(temp[3]) + ', ' + str(rh[3]) + ']\n')

	file.close()

def classify_objects(client, d, t, parameters, parameter_names, parameter_lower_bounds, parameter_lower_bounds1, parameter_upper_bounds, parameter_upper_bounds1):
	for p in parameters:
		if p != "Date":
			try:
				index = parameter_names.index(p)
				if (d.getValueOfParameter(p) == ""):
					break
				value = float(d.getValueOfParameter(p))

				if (value < parameter_lower_bounds[index] or value > parameter_upper_bounds1[index]):
					d.classification = "bad"
				elif (value < parameter_upper_bounds[index] and value > parameter_lower_bounds1[index]):
					if (d.classification == "unknown"):
						d.classification = "good"
				else:
					if (d.classification != "bad"):
						d.classification = "mediocre"
			except:
				pass

	for o in client['objects']:
		if (o.property == t):
			o.history.append(d.classification)
			o.datapoints.append(d)
			continue

	d.classification = "unknown"

def step(client, server):
	if client['data'] == None:
		msg = "{\"action\" : \"steperror\", \"data\": \"No data\" }"
		server.send_message(client, msg)

		return

	if client['counter'] >= len(client['data']):
		msg = "{\"action\" : \"steperror\", \"data\": \"No more data\" }"
		server.send_message(client, msg)

		return

	d = client['data'][client['counter']]

	parameters = d.getParameterNames()
	classify_objects(client, d, u"General", d.getParameterNames(), general.parameter_names, general.parameter_lower_bounds, general.parameter_lower_bounds1, general.parameter_upper_bounds, general.parameter_upper_bounds1)
	classify_objects(client, d, u"Wood", d.getParameterNames(), wood.parameter_names, wood.parameter_lower_bounds, wood.parameter_lower_bounds1, wood.parameter_upper_bounds, wood.parameter_upper_bounds1)
	classify_objects(client, d, u"Copper", d.getParameterNames(), copper.parameter_names, copper.parameter_lower_bounds, copper.parameter_lower_bounds1, copper.parameter_upper_bounds, copper.parameter_upper_bounds1)

	for o in client['objects']:
		msg = "{\"action\" : \"stepresult\", \"date\" : \"" + d.getDate() + "\", \"name\" : \"" + o.name + "\", \"classification\" : \"" + o.history[-1] + "\"}"
		server.send_message(client, msg)

	client['counter'] += 1

def load_file(filename, client):
	data = []

	file_location = "html/uploads/" + filename
	file = open(file_location, "r")
	line = file.readline()
	line = line.rstrip()

	parameter_names = line.split(";")
	parameter_names = parameter_names[:-1]

	line = file.readline()
	while line:
		line = line.rstrip()
		line = line.replace(",", ".")
		parameter_values = line.split(";")
		parameter_values = parameter_values[:-1]

		d = DataPoint(parameter_names, parameter_values)
		data.append(d)

		line = file.readline()


	client['data'] = data
	client['counter'] = 0

def getHistory(server, client, name):
	print "getting history of object ", name
	for o in client['objects']:
		if o.name == name:

			print "object found"
			i = 0

			for h in o.history:
				d = o.datapoints[i]

				msg = "{\"action\" : \"history\", \"date\" : \"" + d.getDate() + "\", \"rh\" : \"" + d.getValueOfParameter("Relative humidity") + "\", \"temp\" : \"" + d.getValueOfParameter("Temperature") + "\", \"classification\" : \"" + h + "\"}"
				print msg
				server.send_message(client, msg)

				i += 1
			break

# Called when a client sends a message
def message_received(client, server, message):
	d = json.loads(message)

	#print d

	if (d['action'] == "create"):
		create_object(d, client)
	elif (d['action'] == "delete"):
		delete_object(d, client)
	elif (d['action'] == "movement"):
		move_object(d, client)
	elif (d['action'] == "rename"):
		rename_object(d, client)
	elif (d['action'] == "getfiles"):
		send_files(client, server)
	elif (d['action'] == "settings"):
		send_settings(client, server, d['name'])
	elif (d['action'] == "savesettings"):
		save_settings(d['name'], d['data'], d['s_type'])
	elif (d['action'] == "loadfile"):
		load_file(d['name'], client)
	elif (d['action'] == "step"):
		step(client, server)
	elif (d['action'] == "history"):
		getHistory(server, client, d['name'])

	"""
	if len(message) > 200:
		message = message[:200]+'..'
	print("Client(%d) said: %s" % (client['id'], message))
	"""


PORT=9001
server = WebsocketServer(PORT)
server.set_fn_new_client(new_client)
server.set_fn_client_left(client_left)
server.set_fn_message_received(message_received)
server.run_forever()